import java.awt.*;
import java.util.*;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.geom.*;


////////////////////
// COMPILATION LINE
//-cp .;vbp.jar;QTJava.zip $BaseName

////////////////////
// RUN LINE
//-classpath .;vbp.jar;QTJava.zip $FileName

public  class pointilism extends Frame {
    
    // window sizers
    static int kWidth ;
    static int kHeight ;
    
    // Image handlers and GUI
    static PixelSource ps;
    static ImageWrangler iw;
    static pointilism pointGUI;
    static Image myImage;
    
    // our looping boolean
    static boolean running = true;
    
    // length of sticks
    static double halfStick = 2.0;

    static double frameBorder = 10.0;
    
    // constructor
    pointilism() {
        kWidth = 640;
        kHeight = 480;
    }
    
    // main function
    public static void main(String args[]) {
        
        // create and size the window
        pointGUI = new pointilism();
       	pointGUI.setSize(kWidth + (int)(frameBorder * 2), kHeight + (int)(frameBorder * 2));
        
       	// set the visibility and layout of the window
        pointGUI.show();
        pointGUI.toFront();
        pointGUI.setLayout (null);
        
        // get our video source
        ps = new PixelSource(kWidth,kHeight);
        
        // get our image handler
        iw = new ImageWrangler(ps.vidWidth,kHeight,ps.getMasks());
        
        //add a listener for shutting the window, give it a method to call (thisWindowClosing)
        pointGUI.addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent e) {
                pointGUI.thisWindowClosing(e);
            }
        });
        
        // always loop
        while (running) {
            
            // grab a frame
            LookAtAFrame();
            
            // paint the frame
            pointGUI.repaint();
        }
    }
    
    
    
    static void LookAtAFrame() {
        
        ps.grabFrame();

    }
    
    public void update(Graphics g){
        paint(g);
    }
    
    public void paint(Graphics g) {

        // convert our graphics
        Graphics2D g2 = (Graphics2D)g;
        
        // int array for grabbing colors
        int rgb[] = new int[4];

        // set the screen composite mode
        Composite c;
        
        // drawing data
        Ellipse2D epoint;
        int hor, ver;
        
        // figure out some horizontal and vertical pixel averages
        for (int linecount = 0; linecount < 1000; linecount++) {
        
            do {
            // choose which pixel to use
            hor = (int)(Math.random() * 640.0);
            ver = (int)(Math.random() * 480.0);
                
            // pull down the pixels
            rgb = ps.getPixel(hor, ver);
            } while ( (rgb[0] == 0) && (rgb[1] == 0) && (rgb[2] == 0));
            
            // set the alpha composite
            c = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, .8f);
            g2.setComposite(c);
            
            // now draw our line across the image
            Color lineColor = new Color(rgb[0], rgb[1], rgb[2]);
          
            // reset the pixel
            ps.setPixel(hor, ver, 0);
            
                // figure out our line
            double x1 = hor - halfStick + frameBorder;
            double y1 = ver - halfStick + frameBorder;
        
            // draw the point            
            epoint = new Ellipse2D.Double(x1, y1, halfStick * 2, halfStick * 2);
            g2.setPaint(lineColor);
            g2.fill(epoint);        
        }
    }
    
    void thisWindowClosing(java.awt.event.WindowEvent e) {
        
        // tell our main loop to stop
        running = false;
        
        // dispose of our objects and quit
        System.out.println ("quit");
        dispose();
        ps.killSession();
        System.exit(5);
    }
}

class colorType {
    int r;
    int g;
    int b;
    int tolerance;
    int occurances;
    int dr;
    int dg;
    int db;
    
    colorType() {
        
        // initialize some of our variables
        r = 0;
        g = 0;
        b = 0;
        
        tolerance = 0;
        occurances = 0;
        dr = 0;
        dg = 0;
        db = 0;
    }
    
    colorType(int _r, int _g, int _b) {
        setColor(_r, _g, _b);
    }
    
    void setColor(int _r, int _g, int _b) {
        r = _r;
        g = _g;
        b = _b;
        
        occurances = 1;
        
        dr = _r;
        dg = _g;
        db = _b;
    }
    
    void setTolerance(int _tolerance) {
        tolerance = _tolerance;
    }
    
    boolean colorEquals(colorType _color) {
        
        boolean colormatches = true;
        int colorDelta = 0;
        
        colorDelta = Math.abs(r - _color.r);
        colorDelta += Math.abs(g - _color.g);
        colorDelta += Math.abs(b - _color.b);
        
        if (colorDelta > tolerance) {
            colormatches = false;
        }
        
        return colormatches;
    }
    
    boolean colorLessThan(colorType _color) {
        
        // compares the _color to the current color
        // in terms of brightness
        int localBright = (r + g + b) / 3;
        int remoteBright = (_color.r + _color.g + _color.b) / 3;
        
        return (remoteBright < localBright);
    }
    
    boolean colorGreaterThan(colorType _color) {
        int localBright = (r + g + b) / 3;
        int remoteBright = (_color.r + _color.g + _color.b) / 3;
        
        return (remoteBright > localBright);
        
    }
    
    void addColor(colorType _color) {
        // add these color values to this type
        
        occurances++;
        
        dr += _color.r;
        dg += _color.g;
        db += _color.b;
        
    }
    
    int getRshade() {
        
        return (dr / occurances);
        //return r;
    }
    
    int getGshade() {
        
        return (dg / occurances);
        //return g;
    }
    
    int getBshade() {
        
        return (db / occurances);
        //return b;
    }
}



